/* lirz.c               Thomas R. Nicely          2008.10.03.0530
 *
 * Freeware copyright (c) 2008 Thomas R. Nicely
 * <http://www.trnicely.net>. No warranties expressed or implied.
 * Distributed under the terms of the GNU GPL and GNU FDL licenses;
 * see <http://www.gnu.org/licenses/licenses.html>. Source,
 * binaries, and license terms available upon request.
 *
 * DESCRIPTION:
 *
 * Calculates the logarithmic integral functions Li(x), HL2(x), HL3(x),
 * HL4(x), and R(x) to up to 96 significant decimal digits (default is
 * 38 SDD), using MPFR. If GMP and MPFR are not available, only the
 * long double results are returned (19 SDD). Otherwise, both sets
 * of results are printed for comparison. If x < 2, artificial values
 * of zero are returned.
 *
 * See the library file trn.c (required for compilation) for the actual
 * computational routines mpfrLIRZ, ldLogInt, and ldRPCF.
 *
 * SYNTAX:
 *
 * lirz2 x [BitsPrec]
 *
 * where BitsPrec is the number of bits of precision [32 <= BitsPrec <= 320],
 * the default being 128 bits (38 significant decimal digits).
 *
 * Developed with version 4.21 of GMP and version 2.2.1 of MPFR. Compile
 * with a command line such as
 *
 * gcc lirz.c trn.c -lm -lmpfr -lgmp -olirz.exe
 *
 * If GMP and MPFR are unavailable, comment out their #includes before
 * compiling, and omit the -lmpfr and -lgmp links. In this case, only
 * long double precision (19 significant digits) will be returned
 * (MSVC will only produce 16 significant digits, since long doubles
 * are cast to doubles).
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdint.h>  /* comment out for Borland C 5.51 */
#include <signal.h>
#include <gmp.h>
#include <mpfr.h>
#include "trn.h"

static const char signature[]=
  "\n __lirz2.c__Version 2008.10.03.0530__Freeware copyright (c) 2008"
  "\n Thomas R. Nicely <http://www.trnicely.net>. No warranties expressed"
  "\n or implied. Distributed under the terms of the GNU GPL and GNU FDL"
  "\n licenses; see <http://www.gnu.org/licenses/licenses.html>. Source,"
  "\n binaries, and license terms available upon request.\n";

/**********************************************************************/
int main(int argc, char **argv)
{
char sz[256];
int iLocalPrec, j, iDP;
double t1, t2;
long double ld, ldx, ldLix, ldHL2, ldHL3, ldHL4, ldR;
#ifdef __MPFR__
mpf_t mpf;
mpfr_t mpfrx, mpfrLi, mpfrHL2, mpfrHL3, mpfrHL4, mpfrR, mpfr;
#endif

if(argc < 2)
  {
  fprintf(stderr, "\n SYNTAX: %s x [BitsPrec]", argv[0]);
  exit(EXIT_FAILURE);
  }

#ifdef __MPFR__

iLocalPrec=128;
if(argc > 2)
  {
  j=atoi(argv[2]);
  if((j > 31) && (j < 321))iLocalPrec=j;
  }
mpfr_set_default_prec(iLocalPrec);

mpfr_init_set_ui(mpfrx, 0, GMP_RNDN);
mpfr_init_set_ui(mpfrLi, 0, GMP_RNDN);
mpfr_init_set_ui(mpfrHL2, 0, GMP_RNDN);
mpfr_init_set_ui(mpfrHL3, 0, GMP_RNDN);
mpfr_init_set_ui(mpfrHL4, 0, GMP_RNDN);
mpfr_init_set_ui(mpfrR, 0, GMP_RNDN);
mpfr_init_set_ui(mpfr, 0, GMP_RNDN);
mpf_init2(mpf, iLocalPrec);

/* If MPFR and GMP are available, the results will be printed
   from both MPFR and long double calculations. */

iDP=floor(iLocalPrec*log(2)/log(10)) - 1;  /* Display precision */

mpfr_init_set_str(mpfrx, argv[1], 10, GMP_RNDN);
mpfr_get_f(mpf, mpfrx, GMP_RNDN);
gmp_printf("\n x  =%.*Fe (MPFR)", iDP, mpf);

mpfrLIRZ(mpfrx, mpfrLi, mpfrHL2, mpfrHL3, mpfrHL4, mpfrR, iLocalPrec);
ldx=___strtold(argv[1], NULL);
ldLix=ldLogInt(ldx, &ldHL2, &ldHL3, &ldHL4);

mpfr_get_f(mpf, mpfrLi, GMP_RNDN);
gmp_printf("\n Li =%.*Fe (MPFR)", iDP, mpf);
mpfr_set_ld(mpfr, ldLix, GMP_RNDN);
mpfr_get_f(mpf, mpfr, GMP_RNDN);
gmp_printf("\n Li =%.18Fe (LD)", mpf);

mpfr_get_f(mpf, mpfrHL2, GMP_RNDN);
gmp_printf("\n HL2=%.*Fe (MPFR)", iDP, mpf);
mpfr_set_ld(mpfr, ldHL2, GMP_RNDN);
mpfr_get_f(mpf, mpfr, GMP_RNDN);
gmp_printf("\n HL2=%.18Fe (LD)", mpf);

mpfr_get_f(mpf, mpfrHL3, GMP_RNDN);
gmp_printf("\n HL3=%.*Fe (MPFR)", iDP, mpf);
mpfr_set_ld(mpfr, ldHL3, GMP_RNDN);
mpfr_get_f(mpf, mpfr, GMP_RNDN);
gmp_printf("\n HL3=%.18Fe (LD)", mpf);

mpfr_get_f(mpf, mpfrHL4, GMP_RNDN);
gmp_printf("\n HL4=%.*Fe (MPFR)", iDP, mpf);
mpfr_set_ld(mpfr, ldHL4, GMP_RNDN);
mpfr_get_f(mpf, mpfr, GMP_RNDN);
gmp_printf("\n HL4=%.18Fe (LD)", mpf);

mpfr_get_f(mpf, mpfrR, GMP_RNDN);
gmp_printf("\n R  =%.*Fe (MPFR)", iDP, mpf);
mpfr_set_ld(mpfr, ldRPCF(ldx), GMP_RNDN);
mpfr_get_f(mpf, mpfr, GMP_RNDN);
gmp_printf("\n R  =%.18Fe (LD)", mpf);

#endif  /* MPFR */

#ifndef __MPFR__

/* If MPFR and GMP are not available, only the long double results
   (19 SDD, except 16 SDD under MSVC) can be calculated and printed. */

ldx=___strtold(argv[1], NULL);
__szLD(sz, ldx, "%.18Le");
printf("\n x  =%s", sz);
ldLix=ldLogInt(ldx, &ldHL2, &ldHL3, &ldHL4);
__szLD(sz, ldLix, "%.18Le");
printf("\n Li =%s", sz);
__szLD(sz, ldHL2, "%.18Le");
printf("\n HL2=%s", sz);
__szLD(sz, ldHL3, "%.18Le");
printf("\n HL3=%s", sz);
__szLD(sz, ldHL4, "%.18Le");
printf("\n HL4=%s", sz);
ldR=ldRPCF(ldx);
__szLD(sz, ldR, "%.18Le");
printf("\n R  =%s", sz);

#endif  /* no MPFR */

printf("\n");
return(EXIT_SUCCESS);
}
/**********************************************************************/
